double T0;
double dtau2;
double m;
double a2;
double b;
double R;
double Z;
double H;
double Hz;
double eps2;
double eps3;
double chi2;
double chi3;
double Rdot;
double Zdot;
double Hdot;
double Hzdot;
double eps2dot;
double eps3dot;
double chi2dot;
double chi3dot;
double Rddot;
double Zddot;
double Hddot;
double Hzddot;
double eps2ddot;
double eps3ddot;
double chi2ddot;
double chi3ddot;

double E(double m, double pt)
{
	return(sqrt(m*m+pt*pt));
}

double Spectra(double pt, double alpha)
{
	double e  = E(m,pt);
	double px = pt*cos(alpha);
	double py = pt*sin(alpha);
	double pz = 0.1;
	
	// Nyeregponti koordináták Descartes-rendszerben
	double rx_s = (px*R*R)/(e*R*R*(1+a2)-2*(T0*b-a2*e)*H);
	double ry_s = (py*R*R)/(e*R*R*(1+a2)-2*(T0*b-a2*e)*H);
	double rz_s = (pz*Z*Z)/(e*Z*Z*(1+a2)-2*(T0*b-a2*e)*Hz);
	
	// Nyeregponti koordináták hengerkoordináta-rendszerben
	double r_s   = sqrt(rx_s*rx_s+ry_s*ry_s);
	double phi_s = atan(ry_s/rx_s);
	double z_s   = rz_s;
	 
	// Phi deriváltak, ezeket a nyeregpontban kell venni
	double dxPhi   = r_s/H*((1+chi2)*cos(phi_s)+chi3*cos(2*phi_s));
	double dyPhi   = r_s/H*((1-chi2)*sin(phi_s)-chi3*sin(2*phi_s));
	double dzPhi   = z_s/Hz;
	double dx2Phi  = 1/H*(1+chi2+chi3/2*(3*cos(phi_s)-cos(3*phi_s)));
	double dy2Phi  = 1/H*(1-chi2+chi3/2*(3*cos(phi_s)+cos(3*phi_s)));
	double dz2Phi  = 1/Hz;
	double dxyPhi  = -chi3/2*(3*sin(phi_s)+sin(3*phi_s));
	double dtxPhi  = r_s/H/H*((chi2dot*H-Hdot*(1+chi2))*cos(phi_s)+(chi3dot*H-Hdot*chi3)*cos(2*phi_s));
	
	double dt2xPhi = r_s/pow(H,3)*((chi2ddot*H*H-Hddot*H*(1+chi2)-2*chi2dot*H*Hdot+2*Hdot*Hdot*(1+chi2))*cos(phi_s)
	+(chi3ddot*H*H-Hddot*H*(1+chi3)-2*chi3dot*H*Hdot+2*Hdot*Hdot*(1+chi3))*cos(2*phi_s));
	
	double dtyPhi  = r_s/H/H*((chi2dot*H-Hdot*(1-chi2))*sin(phi_s)-(chi3dot*H-Hdot*chi3)*sin(2*phi_s));
	
	double dt2yPhi = r_s/pow(H,3)*((chi2ddot*H*H-Hddot*H*(1-chi2)-2*chi2dot*H*Hdot+2*Hdot*Hdot*(1-chi2))*sin(phi_s)
	+(chi3ddot*H*H-Hddot*H*(1+chi3)-2*chi3dot*H*Hdot+2*Hdot*Hdot*(1+chi3))*sin(2*phi_s));
	
	double dtzPhi  = -z_s*Hzdot/Hz/Hz;
	double dt2zPhi = (Hzddot*Hz-2*Hzdot*Hzdot)/pow(Hz,3);
	double dx2tPhi = (chi2dot*H-Hdot*(1+chi2))*cos(2*phi_s)+(chi3dot*H-Hdot*chi3)*(cos(2*phi_s)*cos(phi_s)-2*sin(2*phi_s)*sin(phi_s));
	double dy2tPhi = (chi2dot*H-Hdot*(1-chi2))-(chi3dot*H-Hdot*chi3)*(sin(phi_s)*sin(2*phi_s)+2*cos(phi_s)*cos(2*phi_s));
	double dz2tPhi = -Hzdot/Hz/Hz;
	double dxytPhi = (chi3dot*H-Hdot*chi3)*(cos(2*phi_s)*sin(phi_s)-2*cos(phi_s)*sin(2*phi_s));
	
	// Gamma és deriváltjai, ezeket a nyeregpontban kell venni
	double gamma    = sqrt(1+(dxPhi)*dxPhi+(dyPhi)*(dyPhi)+(dzPhi)*(dzPhi));
	double gamma2   = gamma*gamma;
	double dxgamma  = (dxPhi*dx2Phi+dyPhi*dxyPhi)/gamma;
	double dygamma  = (dyPhi*dy2Phi+dxPhi*dxyPhi)/gamma;
	double dzgamma  = (dzPhi*dz2Phi)/gamma;
	double dt2gamma = (gamma*(dtxPhi*dtxPhi+dxPhi*dt2xPhi+dtyPhi*dtyPhi+dyPhi*dt2yPhi+dtzPhi*dtzPhi+dzPhi*dt2zPhi) + 
	(dxPhi*dtxPhi+dyPhi*dtyPhi+dzPhi*dtzPhi)*(dxPhi*dtxPhi+dyPhi*dtyPhi+dzPhi*dtzPhi)/gamma)/gamma2;
	double dtxgamma = ((dtxPhi*dx2Phi+dxPhi*dx2tPhi+dtyPhi*dxyPhi+dyPhi*dxytPhi)*gamma - 
	(dxPhi*dx2Phi+dyPhi*dxyPhi)*(dxPhi*dtxPhi+dyPhi*dtyPhi+dzPhi*dtzPhi)/gamma)/gamma2;
	double dtygamma = ((dtyPhi*dy2Phi+dyPhi*dy2tPhi+dtxPhi*dxyPhi+dxPhi*dxytPhi)*gamma - 
	(dyPhi*dy2Phi+dxPhi*dxyPhi)*(dxPhi*dtxPhi+dyPhi*dtyPhi+dzPhi*dtzPhi)/gamma)/gamma2;
	double dtzgamma = ((dtzPhi*dz2Phi+dzPhi*dz2tPhi)*gamma - (dzPhi*dz2Phi)*(dxPhi*dtxPhi+dyPhi*dtyPhi+dzPhi*dtzPhi)/gamma)/gamma2;
	
	// p_mu u^mu és deriváltjai, nyeregpontban van
	double p_mu_u_mu   = e*gamma - px*dxPhi-py*dyPhi-pz*dzPhi;
	double dxp_mu_u_mu = e*dxgamma - px*dx2Phi-py*dxyPhi;
	double dyp_mu_u_mu = e*dygamma - px*dxyPhi-py*dy2Phi;
	double dzp_mu_u_mu = e*dzgamma - pz*dz2Phi;
	
	// p_mu u^mu második deriváltjai, nyeregpontban van
	double dt2p_mu_u_mu = e*dt2gamma - px*dt2xPhi - py*dt2yPhi - pz*dt2zPhi;
	double dx2p_mu_u_mu = (((dx2Phi*dx2Phi+dxyPhi*dxyPhi)*gamma)-((dxPhi*dx2Phi+dyPhi*dxyPhi)*(dxPhi*dx2Phi+dyPhi*dxyPhi)/gamma))/gamma2;
	double dy2p_mu_u_mu = (((dy2Phi*dy2Phi+dxyPhi*dxyPhi)*gamma)-((dxPhi*dxyPhi+dyPhi*dy2Phi)*(dxPhi*dxyPhi+dyPhi*dy2Phi)/gamma))/gamma2;
	double dz2p_mu_u_mu = (((dz2Phi*dz2Phi)*gamma)-((dzPhi*dz2Phi)*(dzPhi*dz2Phi)/gamma))/gamma2;
	double dxyp_mu_u_mu = (((dxyPhi*dy2Phi+dxyPhi*dx2Phi)*gamma)-((dxPhi*dxyPhi+dyPhi*dy2Phi)*(dxPhi*dx2Phi+dyPhi*dxyPhi)/gamma))/gamma2;
	double dxzp_mu_u_mu = ((dxPhi*dx2Phi+dxyPhi*dyPhi))*dzPhi*dz2Phi/pow(gamma,1.5);
	double dyzp_mu_u_mu = ((dyPhi*dy2Phi+dxyPhi*dxPhi))*dzPhi*dz2Phi/pow(gamma,1.5);
	double dxtp_mu_u_mu = e*dtxgamma-px*dx2tPhi-py*dxytPhi;
	double dytp_mu_u_mu = e*dtygamma-px*dxytPhi-py*dy2tPhi;
	double dztp_mu_u_mu = e*dtzgamma-pz*dz2tPhi;
	
	// skálafüggvény és deriváltjai, a nyeregpontban kell venni
	double s     = r_s*r_s/R/R*(1+eps2*cos(phi_s)+eps3*cos(3*phi_s));
	double dxs   = 2*r_s/R/R*((1+eps2)*cos(phi_s)+eps3*cos(2*phi_s));
	double dys   = 2*r_s/R/R*((1-eps2)*sin(phi_s)-eps3*sin(2*phi_s));
	double dzs   = 2*z_s/Z;
	double dx2s  = 2/R/R*((1+eps2)+eps3/2*(3*cos(phi_s)-cos(3*phi_s)));
	double dy2s  = 2/R/R*((1-eps2)+eps3/2*(3*cos(phi_s)+cos(3*phi_s)));
	double dz2s  = 2/Z/Z;
	double dt2s  = r_s*r_s*(-(2*(Rddot*R-Rdot*Rdot)-6*R*R*Rdot*Rdot)/pow(R,6)+cos(2*phi_s)*(pow(R,4)*(eps2ddot*R*R+2*eps2dot*R*Rdot
	-2*(eps2dot*R*Rdot+eps2*Rdot*Rdot+eps2*R*Rddot))-4*R*R*R*Rdot*(eps2dot*R*R-2*eps2*R*Rdot))/pow(R,8)+
	cos(3*phi_s)*(pow(R,4)*(eps3ddot*R*R+2*eps3dot*R*Rdot
	-2*(eps3dot*R*Rdot+eps3*Rdot*Rdot+eps3*R*Rddot))-4*R*R*R*Rdot*(eps3dot*R*R-2*eps3*R*Rdot))/pow(R,8));
	double dxts  = 2*r_s*(((1+eps2)*2*R*Rdot-eps2dot*R*R)/pow(R,4)*cos(phi_s)+(eps3*2*R*Rdot+eps3dot*R*R)/pow(R,4)*cos(2*phi_s));
	double dyts  = 2*r_s*(((1-eps2)*2*R*Rdot-eps2dot*R*R)/pow(R,4)*sin(phi_s)-(eps3*2*R*Rdot+eps3dot*R*R)/pow(R,4)*sin(2*phi_s));
	double dzts  = -4*z_s*(Zdot/Z/Z);
	double dxys  = -eps3/2/R*(3*sin(phi_s)+sin(3*phi_s));
	
	// A Young-tétel miatt szimmetrikus determináns elemei. Mindegyik R_ik = 1/R_ik^2 alakúnak értendő.
	// Az egész a nyeregpontban van!
	double Rtt = 1/dtau2+((1+a2*s)/T0*dt2p_mu_u_mu+(p_mu_u_mu*2/T0-b)*dt2s)/gamma2;
	double Rtx = ((1+a2*s)/T0*dxtp_mu_u_mu + (p_mu_u_mu*a2/T0-b)*dxts)/gamma2;
	double Rty = ((1+a2*s)/T0*dytp_mu_u_mu + (p_mu_u_mu*a2/T0-b)*dyts)/gamma2;
	double Rtz = ((1+a2*s)/T0*dztp_mu_u_mu + (p_mu_u_mu*a2/T0-b)*dzts)/gamma2;
	double Rxx = (1+a2*s)/T0*dx2p_mu_u_mu  + (p_mu_u_mu*a2/T0-b)*dx2s;
	double Ryy = (1+a2*s)/T0*dy2p_mu_u_mu  + (p_mu_u_mu*a2/T0-b)*dy2s;
	double Rzz = (1+a2*s)/T0*dz2p_mu_u_mu  + (p_mu_u_mu*a2/T0-b)*dz2s;
	double Rxy = (1+a2*s)/T0*dxyp_mu_u_mu  + (p_mu_u_mu*a2/T0-b)*dxys;
	double Rxz = (1+a2*s)/T0*dxzp_mu_u_mu;
	double Ryz = (1+a2*s)/T0*dyzp_mu_u_mu;
	double detR_mu_nu = Rtt*Rxx*Ryy*Rzz-Rtt*Rxx*Ryz*Ryz-Rtt*Rxy*Rxy*Rzz+2*Rtt*Rxy*Rxz*Ryz-Rtt*Rxz*Rxz*Ryy-Rtx*Rtx*Ryy*Rzz
	+Rtx*Rtx*Ryz*Ryz+2*Rtx*Rty*Rxy*Rzz-2*Rtx*Rty*Rxz*Ryz-2*Rtx*Rtz*Rxy*Ryz+2*Rtx*Rtz*Rxz*Ryy-Rty*Rty*Rxx*Rzz+Rty*Rty*Rxz*Rxz+
	2*Rty*Rtz*Rxx*Ryz-2*Rty*Rtz*Rxy*Rxz-Rtz*Rtz*Rxx*Ryy+Rtz*Rtz*Rxy*Rxy;
	
	// Spektrum
	double g           = 1;
	double prefactor   = g/pow(2*M_PI,3);
	double exponential = 1/exp(p_mu_u_mu/T0*(1+a2*s)-b*s);
	double detR_factor = sqrt(1/detR_mu_nu);
	double spectra     = prefactor*p_mu_u_mu*exponential;
	
	spectra = pow(10,6)*spectra;
	return( spectra );	
}
